import distutils
import http
import os
import string
from datetime import datetime, timezone, timedelta
from distutils import util
import random

import flask
import pytest
from bson import json_util, ObjectId
from flask import Flask, request, jsonify, Blueprint, make_response, send_file
from pymongo import MongoClient
from werkzeug.datastructures import FileStorage
from werkzeug.utils import secure_filename

from config.utils import Utils
from modele.contenu_dao import ContenuDao
from modele.demande_pubDao import DemandePubDao
from modele.encours_dao import EncoursDao
from modele.evaluation_dao import EvaluationDao
from modele.utilisateur_dao import UtilisateurDao

import json

from flask import Flask
from flask_cors import CORS, cross_origin
from flask_restx import Api, Resource, Namespace, namespace

from flask_jwt_extended import create_access_token, get_jwt, set_access_cookies, unset_jwt_cookies
from flask_jwt_extended import get_jwt_identity
from flask_jwt_extended import jwt_required
from flask_jwt_extended import JWTManager

app = Flask(__name__)
cors = CORS(app)
app.config['CORS_HEADERS'] = 'Content-Type'

contenu_dao = ContenuDao()
utilisateur_dao = UtilisateurDao()
demande_pub_Dao = DemandePubDao()
evaluation_dao = EvaluationDao()
en_cours_dao = EncoursDao()

client = MongoClient('localhost', 27017)

base_donnees = client.squelette

app.config["JWT_SECRET_KEY"] = "l'ordinateur-c'est-magique"  # Change this!
jwt = JWTManager(app)

## *********************************************************************************************************************
## Entrées spécifique pour swagger
blueprint = Blueprint('api', __name__)
api = Api(blueprint, doc='/doc/')

app.register_blueprint(blueprint)



@app.route('/test')
@cross_origin()
def hello_world():
    return '<!DOCTYPE html> <html  dir="ltr" lang="fr" xml:lang="fr"><head><link rel="stylesheet" type="text/css" ' \
           'href="https://celene.univ-orleans.fr/theme/yui_combo.php?rollup/3.17.2/yui-moodlesimple-min.css" ' \
           '/><script id="firstthemesheet" type="text/css">/** Required in order to fix style inclusion problems in ' \
           'IE with YUI **/</script><link rel="stylesheet" type="text/css" ' \
           'href="https://celene.univ-orleans.fr/theme/styles.php/celeneboost/1617792745_1614585414/all" ' \
           '/></head><body><div role="main"><span id="maincontent"></span><form method="post" ' \
           'action="https://celene.univ-orleans.fr/question/preview.php?id=7920206&amp;previewid=696734&amp;courseid' \
           '=4997&amp;variant=1&amp;correctness=1&amp;marks=1&amp;markdp=2&amp;feedback=1&amp;generalfeedback=1&amp' \
           ';rightanswer=1&amp;history=0" enctype="multipart/form-data" id="responseform"><div><input type="hidden" ' \
           'name="sesskey" value="zESgJU3xYb" /><input type="hidden" name="slots" value="1" /><input type="hidden" ' \
           'name="scrollpos" value="" id="scrollpos" /></div><div id="q1" class="que truefalse deferredfeedback ' \
           'notyetanswered"><div class="info"> <h3 class="no">Question <span class="qno">1</span></h3>      <div ' \
           'class="state">Pas encore répondu</div> <div class="grade">Noté sur 10,00</div></div><div ' \
           'class="content"><div class="formulation clearfix"> <h4 class="accesshide">Texte de la question</h4> ' \
           '<input type="hidden" name="q696734:1_:sequencecheck" value="1" /><div class="qtext"> <p>ertert</p> </div> ' \
           '<div class="ablock"> <div class="prompt">Sélectionnez une réponse :</div> <div class="answer"><div ' \
           'class="r0"><input type="radio" name="q696734:1_answer" value="1" id="q696734:1_answertrue" /><label ' \
           'for="q696734:1_answertrue" class="m-l-1">Vrai</label> </div><div class="r1"><input type="radio" ' \
           'name="q696734:1_answer" value="0" id="q696734:1_answerfalse" /><label for="q696734:1_answerfalse" ' \
           'class="m-l-1">Faux</label> </div></div></div></div></div></div></form></div></body></html> '


@jwt.unauthorized_loader
def my_invalid_token_callback(expired_token):
    return jsonify(code="dave", err="I can't let you do that"), 401


@app.after_request
def refresh_expiring_jwts(response):
    try:
        exp_timestamp = get_jwt()["exp"]
        now = datetime.now(timezone.utc)
        target_timestamp = datetime.timestamp(now + timedelta(minutes=30))
        if target_timestamp > exp_timestamp:
            access_token = create_access_token(identity=get_jwt_identity())
            set_access_cookies(response, access_token)
        return response
    except (RuntimeError, KeyError):
        # Case where there is not a valid JWT. Just return the original respone
        return response


## *********************************************************************************************************************
## Test de manipulation des images

image = api.namespace('images', description="pour les champs qui le nécessitent, permet de récupérer des images")
@app.route("/getImage/<nom_image>", methods=['GET'])
def monPetitProutney(image):
    #return "Mais oui, les enfantches", 200
    return send_file("/home/passerat/Stage/volet_navigation/src/assets/img/croix.png")

users = api.namespace('utilisateurs', description='Entrées liées à la gestion utilisateur')

## *********************************************************************************************************************
## Entrées liées à la gestion utilisateur
@users.route('/newUser')
@users.doc(params={'login': "le nom de l'utilisateur à créer"})
@users.doc(description="Création d'un nouvel utilisateur")
class CreerUtilisateur(Resource):
    @users.response(201, 'Utilisateur créé')
    @users.response(409, 'Utilisateur existant')
    ## Création d'un nouvel utilisateur
    def post(self):
        if utilisateur_dao.presence_utilisateur(request.json["login"]) == 0:
            utilisateur_dao.ajouter_utilisateur(request.json["login"])
            return "Utilisateur créé", 201
        else:
            return "Utilisateur existant", 409


@users.route('/checkUser')
@users.doc(params={'login': "le nom de l'utilisateur à vérifier"})
@users.doc(description="vérification qu'un utilisateur est enrengistré dans la base")
class VerifierUtilisateur(Resource):
    @users.response(200, "Vérification de l'utilisateur effectuée")
    @users.response(400, "Vérification de l'utilisateur ratée")
    ## vérification qu'un utilisateur est enrengistré dans la base
    def get(self):

        if utilisateur_dao.presence_utilisateur(request.args.get("login")):
            try:
                identity = get_jwt_identity()

                access_token = create_access_token(identity=identity)
            except RuntimeError:
                randomChaine = (''.join(random.choice(string.ascii_letters + string.digits) for _ in range(10)))

                access_token = create_access_token(identity=request.args.get("login"))
                response = jsonify({"msg": "login successful"})

                return make_response(jsonify(access_token=access_token), 200)
        else:
            return "L'utilisateur n'existe pas", 400


@users.route("/logout")
class Logout(Resource):
    @jwt_required()
    def get(self):
        response = jsonify({"msg": "logout successful"})
        return response


@users.route('/changePassword')
@users.doc(description="changement du mot de passe d'un utilisateur")
class ChangerMdp(Resource):
    @users.response(200, "Mot de Passe changé")
    @users.response(409, 'Erreur changement mot de passe')
    ## changement du mot de passe d'un utilisateur
    @jwt_required()
    def patch(self):
        pass


@users.route('/auteursExistants')
@users.doc(description="récupération de tous les utilisateurs en base")
class RecupererTousLesUtilisateurs(Resource):
    @users.response(200, "Utilisateurs récupérés")
    ## récupération de tous les utilisateurs en base
    @jwt_required()
    def get(self):
        return json.loads(json.dumps(list(utilisateur_dao.get_all_utilisateurs()), default=json_util.default)), 200


# Fonction (pour le moment) uniquement utilisée pour les tests
@users.route('/supprimerUser/<nom_utilisateur>')
@users.doc(description="récupération de tous les utilisateurs en base")
class Supprimerutilisateur(Resource):
    @users.response(200, "Utilisateur supprimé")
    @users.response(409, 'Utilisateur inconnu')
    ## récupération de tous les utilisateurs en base
    @jwt_required()
    def delete(self, nom_utilisateur):
        if utilisateur_dao.presence_utilisateur(nom_utilisateur):
            utilisateur_dao.supprimer_utilisateur(nom_utilisateur)
            return "Utilisateur supprimé", 200
        else:
            return "Utilisateur inconnu", 409


contenus = api.namespace('contenus', description='Entrées liées à la gestion des contenus')


## *********************************************************************************************************************
## Entrées liées à la gestion des contenus

@contenus.route("/addContenu")
@contenus.doc(params={'auteur': "(l'auteur qui ajoute le contenu"})
@contenus.doc(params={'contenu_a_ecrire': " le contenu à écrire"})
@contenus.doc(params={'titre': " le contenu à écrire"})
@contenus.doc(params={'tags': " le contenu à écrire"})
@contenus.doc(description='ajouter un nouveau contenu')
class AjouterContenu(Resource):
    @contenus.response(201, "Contenu créé")
    ## ajouter un nouveau contenu
    @jwt_required()
    def post(self):
        auteur = request.json["auteur"]
        contenu_a_ecrire = request.json["contenu_a_ecrire"]
        titre = request.json["titre"]
        tags = request.json["tags"]
        contenu_dao.ajouter_contenu(auteur, contenu_a_ecrire, titre, tags)
        return json.loads(json.dumps((contenu_dao.get_contenu_par_id(contenu_dao.id_dernier_contenu_ajoute())),
                                     default=json_util.default)), 201


@contenus.route("/uploadContenu/<auteur>")
@contenus.doc(params={'auteur': "(l'auteur qui ajoute le contenu"})
@contenus.doc(description='ajouter un nouveau contenu')
class UploaderContenu(Resource):
    @contenus.response(201, "Contenu importé")
    ## ajouter un nouveau contenu
    @jwt_required()
    def post(self, auteur):
        file = request.files['file']
        contenu_dao.importer_contenu(auteur, file)
        return json.loads(json.dumps((contenu_dao.get_contenu_par_id(contenu_dao.id_dernier_contenu_ajoute())),
                                     default=json_util.default)), 201


@contenus.route("/incrementerPopularite")
@contenus.doc(params={'id_contenu': "l'id du contenu concerné"})
@contenus.doc(params={'version': "la version dont il faut augmenter la popularité"})
@contenus.doc(description="incrémenter la popularité d'une version d'un contenu")
class IncrementerPopularite(Resource):
    @contenus.response(200, "Popularité Modifiée")
    @jwt_required()
    def patch(self):
        id_contenu = json.loads(request.json["id_contenu"])
        contenu_dao.incrementer_popularite(id_contenu["$oid"])
        return "popularité modifiée", 200


# @contenus.route("/incrementerNote")
# @contenus.doc(params={'id_contenu': "l'id du contenu concerné"})
# @contenus.doc(description="incrémenter la note d'un contenu")
# class IncrementerNote(Resource):
#     @contenus.response(200, "Note Modifiée")
#     @jwt_required()
#     def patch(self):
#         id_contenu = json.loads(request.json["id_contenu"])
#         contenu_dao.incrementer_note_contenu(id_contenu["$oid"])
#         return "popularité modifiée", 200


@contenus.route('/ajouterTiers')
@contenus.doc(params={'auteur': "l'auteur qui ajoute le contenu"})
@contenus.doc(params={'contenu_a_ecrire': " le contenu à écrire"})
@contenus.doc(params={'id_tiers': "l'id du tiers à ajouter"})
@contenus.doc(description='ajouter un nouveau contenu tiers pour un auteur')
class AjouterTiers(Resource):
    @contenus.response(201, "Tiers créé")
    ## ajouter un nouveau contenu tiers pour un auteur
    @jwt_required()
    def patch(self):
        id_tiers = json.loads(request.json["id_tiers"])
        contenu_dao.creer_tiers(request.json["auteur"], id_tiers["$oid"], request.json["version"])
        return json.loads(
            json.dumps((contenu_dao.get_contenu_par_id(id_tiers["$oid"])), default=json_util.default)), 201


@contenus.route("/getContenu/<id>")
@contenus.doc(description='récupérer un contenu via son id')
class GetContenuParId(Resource):
    @contenus.response(200, "Contenu récupéré")
    ## récupérer un contenu via son id
    @jwt_required()
    def get(self, id):
        id_contenu = json.loads(id)

        return json.loads(
            json.dumps(list(contenu_dao.get_contenu_par_id(id_contenu["$oid"])), default=json_util.default)), 200


@contenus.route("/getContenuPanier/<auteur>")
@contenus.doc(params={'perso': "remonter les contenus persos ou non (True/False)"})
@contenus.doc(params={'tiers': " remonter les contenus tiers ou non"})
@contenus.doc(params={'enCours': "remonter les contenus en cours ou non"})
@contenus.doc(description="récupérer le contenu du panier d'un auteur")
class GetContenuPanier(Resource):
    @contenus.response(200, "Panier récupéré")
    ## récupérer le contenu du panier d'un auteur
    @jwt_required()
    def get(self, auteur):
        now = datetime.now(timezone.utc)
        target_timestamp = datetime.timestamp(now + timedelta(minutes=30))
        auteur = auteur.replace("_", " ")
        return json.loads(
            json.dumps(contenu_dao.get_contenus(auteur, request.args.get("perso"), request.args.get("tiers"),
                                                request.args.get("enCours")),
                       default=json_util.default)), 200


@contenus.route("/getContenuBanque/<auteur>")
@contenus.doc(description="récupérer le contenu de la banque pour un auteur")
class GetContenuBanque(Resource):
    @contenus.response(200, "Banque récupérée")
    ## récupérer le contenu de la banque pour un auteur
    @jwt_required()
    def get(self, auteur):
        auteur = auteur.replace("_", " ")
        return json.loads(json.dumps(list(contenu_dao.get_contenus_non_auteur(auteur)),
                                     default=json_util.default)), 200


@contenus.route("/deletePerso/<id_perso>")
@contenus.doc(description="supprimer un contenu")
class SupprimerPerso(Resource):
    @contenus.response(200, "Contenu supprimé")
    ## supprimer un contenu
    @jwt_required()
    def delete(self, id_perso):
        id_perso = json.loads(id_perso)
        contenu_dao.supprimer_contenu(id_perso["$oid"])
        return "Contenu supprimé", 200


@contenus.route("/deleteTiers")
@contenus.doc(params={'auteur': "l'auteur qui supprime le contenu"})
@contenus.doc(params={'id_tiers': "le contenu à supprimer"})
@contenus.doc(description="supprimer un tiers pour un auteur")
class SupprimerTiers(Resource):
    @contenus.response(200, "Tiers supprimé")
    ## supprimer un tiers pour un auteur
    @jwt_required()
    def delete(self):
        id_tiers = json.loads(request.args.get("id_tiers"))

        version = contenu_dao.get_version_tiers_avec_id(id_tiers["$oid"], request.args.get('auteur'))
        contenu_dao.supprimer_tiers(id_tiers["$oid"], request.args.get('auteur'), version, request.args.get('note'))
        if request.args.get('auteur') == "Aucun":
            return str(contenu_dao.existe_en_tiers(id_tiers["$oid"])), 200
        else:
            return '1', 200


@contenus.route("/newContenu")
@contenus.doc(
    params={'id_demande_pub': "l'id de la demande de publication à l'origine de la création du nouveau contenu"})
@contenus.doc(params={'auteur': "l'auteur qui ajoute le contenu"})
@contenus.doc(description="créer un contenu à partir d'une demande de publication refusée")
class creer_nouveau_contenu(Resource):
    @contenus.response(201, "Nouveau contenu créé")
    ## créer un contenu à partir d'une demande de publication refusée
    @jwt_required()
    def post(self):
        id_demande_pub = json.loads(request.json["id_demande_pub"])
        contenu_dao.creer_nouveau_contenu(id_demande_pub["$oid"], request.json["auteur"])
        return json.loads(json.dumps((contenu_dao.get_contenu_par_id(contenu_dao.id_dernier_contenu_ajoute())),
                                     default=json_util.default)), 201


@contenus.route("/majContenu")
@contenus.doc(params={'id_contenu_en_cours': "l'id du contenu en cours à mettre à jour"})
@contenus.doc(params={'auteur': "l'auteur qui fait la mise à jour"})
@contenus.doc(description="mettre à jour un contenu (via un contenu en cours directement)")
class maj_contenu(Resource):
    @contenus.response(200, "Contenu maj")
    ## mettre à jour un contenu (via un contenu en cours directement)"
    @jwt_required()
    def patch(self):
        id_contenu_en_cours = json.loads(request.json["id_contenu_en_cours"])
        if request.json["auteur"] == "":
            auteur = None
        else:
            auteur = request.json["auteur"]
        contenu_dao.maj_contenu(id_contenu_en_cours["$oid"], auteur)
        return "Le contenu a été mis à jour", 200


@contenus.route("/gererDemandePub")
@contenus.doc(params={'id_publication': "l'id de la demande de publication concernée"})
@contenus.doc(params={'reponse': "la réponse à la demande de publication (o / n)"})
@contenus.doc(params={'auteur': "l'auteur qui répond à la demande de publication"})
@contenus.doc(description="gerer la reponse à une demande de publication")
class gerer_demande_pub(Resource):
    @contenus.response(200, "Réponse à la publication gérée")
    ## gerer la reponse à une demande de publication
    @jwt_required()
    def patch(self):
        id_publication = json.loads(request.json["id_publication"])
        demande_pub_Dao.gerer_demande_pub(id_publication["$oid"], request.json["reponse"], request.json["auteur"])
        return "Réponse à la publication gérée", 200


@contenus.route("/allContenus")
@contenus.doc(description="récupérer tous les contenus")
class get_tous_les_contenus(Resource):
    @contenus.response(200, "Tous les contenus récupérés")
    ## récupérer tous les contenus
    @jwt_required()
    def get(self):
        return json.loads(json.dumps(list(contenu_dao.get_tous_les_contenus()), default=json_util.default)), 200


@contenus.route("/getTags/<id_contenu>")
@contenus.doc(params={'id_publication': "l'id du contenu pour lequel on veut récupérer des tags"})
@contenus.doc(description="récupérer les tags d'un contenu")
class get_tags_contenu(Resource):
    @contenus.response(200, "Tags récupérés")
    ## récupérer les tags d'un contenu
    @jwt_required()
    def get(self, id_contenu):
        id_contenu = json.loads(id_contenu)
        return json.loads(json.dumps(list(contenu_dao.get_tags_contenu(id_contenu["$oid"])),
                                     default=json_util.default)), 200


@contenus.route("/modifierTags")
@contenus.doc(params={'id_contenu': "l'id du contenu pour lequel on veut récupérer des tags"})
@contenus.doc(params={'tags': "les nouveaux tags du contenu"})
@contenus.doc(description="modifier les tags d'un contenu")
class modifier_tags_contenus(Resource):
    @contenus.response(200, "Tags modifiés")
    ## récupérer les tags d'un contenu
    @jwt_required()
    def patch(self):
        id_contenu = json.loads(request.json["id_contenu"])
        ContenuDao().changer_tags_contenus(id_contenu["$oid"], request.json["tags"])
        return "tags modifiés", 200


@contenus.route("/newTagLibre")
@contenus.doc(params={'auteur': "le créateur du tag"})
@contenus.doc(params={'tag': "Le tag créé"})
@contenus.doc(description="ajouter un tag libre")
class ajouter_tag_libre(Resource):
    @contenus.response(201, "Tag libre créé")
    ## ajouter un tag libre (lié à un contenu ou non)
    @jwt_required()
    def patch(self):

        id_contenu = ""
        ContenuDao().creer_tag_libre(request.json["auteur"], request.json["tag"])
        return "tag créé", 201


@contenus.route("/getTagLibre/<auteur>")
@contenus.doc(params={'auteur': "l'auteur des tags à récupérer"})
@contenus.doc(description="récupérer tous les tags libres d'un auteur")
class recuperer_tags_libres(Resource):
    @contenus.response(200, "Tags libres récupérés")
    ## récupérer tous les tags libres d'un auteur
    @jwt_required()
    def get(self, auteur):
        return ContenuDao().recuperer_all_tags_libres(auteur), 200


@contenus.route("/getTagLibreContenu/<id_contenu>/<auteur>")
@contenus.doc(params={'id_contenu': "le contenu auquel les tags ont été associés"})
@contenus.doc(params={'auteur': "l'auteur des tags à récupérer"})
@contenus.doc(description="récupérer tous les tags libres d'un auteur associés à un contenu particulier")
class recuperer_tags_libres_pour_un_contenu(Resource):
    @contenus.response(200, "Tags libres contenu récupérés")
    ## récupérer tous les tags libres d'un auteur associés à un contenu particulier
    @jwt_required()
    def get(self, id_contenu, auteur):
        id_contenu = json.loads(id_contenu)
        return ContenuDao().recuperer_tags_libres_contenu(id_contenu["$oid"], auteur), 200


@contenus.route("/associerTagLibre")
@contenus.doc(params={'auteur': "le créateur du tag"})
@contenus.doc(params={'id_contenu': "le contenu à associer du tag"})
@contenus.doc(params={'tag': "Le tag concerné"})
class associer_tag_libre(Resource):
    @contenus.response(201, "Tag libre créé")
    ## associer un tag libre à un contenu
    @jwt_required()
    def patch(self):
        print("Couzin")
        id_contenu = json.loads(request.json["id_contenu"])
        ContenuDao().associer_tag_libre(request.json["auteur"], request.json["tag"], id_contenu["$oid"])
        return "tag associe", 201


@contenus.route("/dissocierTagLibre")
@contenus.doc(params={'auteur': "le créateur du tag"})
@contenus.doc(params={'id_contenu': "le contenu à dissocier du tag"})
@contenus.doc(params={'tag': "Le tag concerné"})
class dissocier_tag_libre(Resource):
    @contenus.response(201, "Tag libre créé")
    ## dissocier un tag libre d'un contenu
    @jwt_required()
    def patch(self):
        id_contenu = json.loads(request.json["id_contenu"])
        ContenuDao().dissocier_tag_libre(request.json["auteur"], request.json["tag"], id_contenu["$oid"])
        return "tag dissocié", 201


@contenus.route("/supprimerTagLibre")
@contenus.doc(params={'auteur': "le créateur du tag"})
@contenus.doc(params={'tag': "Le tag à supprimer"})
class supprimer_tag_libre(Resource):
    @contenus.response(201, "Tag libre créé")
    ## supprimer un tag libre
    @jwt_required()
    def delete(self):
        ContenuDao().supprimer_tag_libre(request.args.get("auteur"), request.args.get("tag"))
        return "tag supprimé", 200


@contenus.route("/isContenuAnonyme/<id_contenu>")
@contenus.doc(description="le contenu est-il anonyme ?")
class is_contenu_anonyme(Resource):
    @contenus.response(200, "Résultat pour savoir si le contenu est anonyme ou non")
    ## Résultat pour savoir si le contenu est anonyme ou non
    @jwt_required()
    def get(self, id_contenu):
        return str(contenu_dao.is_anonyme(id_contenu)), 200


@contenus.route("/isMajTiersDisponible/<auteur>")
@contenus.doc(description="l'auteur dispose t'il de maj tiers faisables ?")
class is_maj_tiers_disponible(Resource):
    @contenus.response(200, "Résultat pour savoir si des mises à jours tiers sont disponibles")
    ## Résultat pour savoir si des mises à jours tiers sont disponibles
    @jwt_required()
    def get(self, auteur):
        return str(contenu_dao.is_tiers_maj(auteur)), 200


@contenus.route("/listeMajTiers/<auteur>")
@contenus.doc(description="liste des majs tiers disponibles pour l'auteur")
class liste_maj_tiers(Resource):
    @contenus.response(200, "Liste des Maj Tiers récupérée")
    ## Liste des Maj Tiers récupérée
    @jwt_required()
    def get(self, auteur):
        return json.loads(json.dumps(contenu_dao.liste_tiers_maj(auteur), default=json_util.default)), 200


## gérer la réponse à une maj tiers
@contenus.route("/gererMajTiers")
@contenus.doc(params={'id_tiers': "l'id du tiers concerné par la maj"})
@contenus.doc(params={'reponse': "la réponse à la demande de maj tiers (o / n)"})
@contenus.doc(params={'auteur': "l'auteur qui répond à la demande de maj tiers"})
@contenus.doc(description="gérer la réponse à une maj tiers")
class gerer_maj_tiers(Resource):
    @contenus.response(200, "Maj du tiers gérée")
    ## gérer la réponse à une maj tiers
    @jwt_required()
    def patch(self):

        id_tiers = json.loads(request.json["id_tiers"])

        if request.json["reponse"] == 'o':
            refuse = False
        else:
            refuse = True

        version = contenu_dao.get_version_tiers_avec_id(id_tiers["$oid"], request.json["auteur"])

        contenu_dao.maj_tiers(id_tiers["$oid"], int(version) + 1, request.json["auteur"], refuse)
        return "Tiers géré", 200


## fonction qui permet de renseigner la note d'un contenu
@contenus.route("/noterContenu")
@contenus.doc(params={'auteur': "l'auteur qui met la note"})
@contenus.doc(params={'id_contenu': "l'id du contenu à noter"})
@contenus.doc(params={'version': "la version du contenu concernée par la note "})
@contenus.doc(params={'note': "la note mise (entre 0 et 5)"})
@contenus.doc(description="permet de noter un contenu")
class noter_contenu(Resource):
    @contenus.response(200, "contenu noté")
    @jwt_required()
    def patch(self):
        note = request.json["note"]
        auteur = request.json["auteur"]
        version = request.json["version"]
        id_contenu = json.loads(request.json["id_contenu"])
        moyenne = ContenuDao().noterContenuTiers(id_contenu["$oid"], version, auteur, note)
        return str(moyenne), 200


@contenus.route("/recupererNote")
@contenus.doc(params={'id_contenu': "l'id du contenu déja noté"})
@contenus.doc(params={'version': "la version du contenu concernée par la note "})
@contenus.doc(params={'auteur': "l'auteur de la note"})
@contenus.doc(description="recuperer la note donnée par un auteur à un contenu")
class RecupererNoteDonneAUnContenu(Resource):
    @contenus.response(200, "note du contenu")
    @jwt_required()
    def get(self):
        id_contenu = json.loads(request.args.get("id_contenu"))
        return str(ContenuDao().recupererNoteContenu(id_contenu["$oid"], request.args.get("version"),
                                                     request.args.get("auteur"))), 200

@contenus.route("/recupererNoteGlobale")
@contenus.doc(params={'id_contenu': "l'id du contenu "})
@contenus.doc(params={'version': "la version du contenu concernée par la note "})
@contenus.doc(description="recuperer la note globale d'un contenu (dans une version donnée")
class RecupererNoteDonneAUnContenu(Resource):
    @contenus.response(200, "note du contenu")
    @jwt_required()
    def get(self):
        id_contenu = json.loads(request.args.get("id_contenu"))
        return str(ContenuDao().recupererNoteGlobaleContenu(id_contenu["$oid"], request.args.get("version"))), 200


## récupérer le texte de la version en cours du contenu
@contenus.route("/getTexteContenu/<id_json>")
@contenus.doc(description="récupérer le texte de la version en cours du contenu")
class get_texte_contenu(Resource):
    @contenus.response(200, "Texte du contenu changé pour la version en cours")
    ## récupérer le texte de la version en cours du contenu
    @jwt_required()
    def get(self, id_json):
        id_contenu = json.loads(id_json)
        return Utils().recuperer_contenu_fichier(id_contenu["$oid"]), 200


## récupérer le texte d'une version donnée  du contenu
@contenus.route("/getTexteVersionContenu/<id_json>/<version>")
@contenus.doc(params={'id_json': "l'id du contenu pour lequel il faut récupérer le texte"})
@contenus.doc(params={'version': "la version à récupérer"})
@contenus.doc(description="récupérer le texte d'une version donnée  du contenu")
class get_texte_version_contenu(Resource):
    @contenus.response(200, "Texte du contenu récupéré pour la version renseignée")
    ## récupérer le texte d'une version donnée  du contenu
    @jwt_required()
    def get(self, id_json, version):
        id_contenu = json.loads(id_json)
        return Utils().recuperer_contenu_version_fichier(id_contenu["$oid"], version), 200


@contenus.route("/changerTexte/<id_contenu>")
@contenus.doc(params={'nouveau_texte': "la mise à jour du texte associé au contenu"})
@contenus.doc(description="changer le texte d'un contenu (maj de la version en cours)")
class changer_texte_contenu(Resource):
    @contenus.response(200, "Texte du contenu changé")
    ## changer le texte d'un contenu (maj de la version en cours)
    @jwt_required()
    def patch(self, id_contenu):

        if len(id_contenu) == 12 or len(id_contenu) == 24:
            id_contenu = ObjectId(id_contenu)
        else:
            tmp = json.loads(id_contenu)
            id_contenu = tmp["$oid"]

        Utils().changer_texte_fichier(id_contenu, request.json["nouveau_texte"], request.json["nouveau_titre"])
        return "Texte Maj", 200


en_cours = api.namespace('en_cours', description='Entrées liées à la gestion des contenus en cours')


## *********************************************************************************************************************
## Entrées liées à la gestion des contenus en cours

@en_cours.route("/getTexteContenuEnCours/<id_en_cours>")
@en_cours.doc(description="récupérer le texte d'un contenu en cours")
class GetTexteContenuEnCours(Resource):
    @en_cours.response(200, "Texte du contenu en cours récupéré")
    ## récupérer le texte d'un contenu en cours
    @jwt_required()
    def get(self, id_en_cours):

        if len(id_en_cours) == 24 or len(id_en_cours) == 12:
            id_du_contenu = ObjectId(id_en_cours)
        else:
            tmp = json.loads(id_en_cours)
            id_du_contenu = tmp["$oid"]

        return Utils().recuperer_contenu_fichier(id_du_contenu, True), 200


@en_cours.route("/getEnCours/<id_en_cours>")
@en_cours.doc(description="récupérer un contenu en cours via son id")
class GetContenuEnCours(Resource):
    @en_cours.response(200, "Contenu en cours récupéré")
    ## récupérer un contenu en cours via son id
    @jwt_required()
    def get(self, id_en_cours):
        return json.loads(
            json.dumps(list(en_cours_dao.get_contenu_en_cours_via_id(id_en_cours)), default=json_util.default)), 200


@en_cours.route("/getAuteurOrigineEnCours/<id_en_cours>")
@en_cours.doc(description="récupérer l'auteur d'origine d'un contenu en cours")
class GetAuteurOtigineContenuEnCours(Resource):
    @en_cours.response(200, "Auteur du contenu en cours récupéré")
    ## récupérer l'auteur d'origine d'un contenu en cours
    @jwt_required()
    def get(self, id_en_cours):
        return json.loads(
            json.dumps(list(en_cours_dao.get_auteur_origine_contenu_en_cours(id_en_cours)),
                       default=json_util.default)), 200


@en_cours.route("/isContenuDemandePublication/<id_en_cours>/<auteur>")
@en_cours.doc(
    description="le contenu en cours à t'il fait l'objet d'une demande de publication par l'auteur renseigné ?")
class IsContenuDemandePublication(Resource):
    @en_cours.response(200, "Résultat pour savoir si le contenu en cours a fait l'objet d'une demande de publication")
    ## le contenu en cours à t'il fait l'objet d'une demande de publication par l'auteur renseigné ?
    @jwt_required()
    def get(self, id_en_cours, auteur):
        return str(en_cours_dao.is_en_cours_demande(id_en_cours, auteur)), 200


@en_cours.route("/supprimerEnCoursParId/<id_en_cours>")
@en_cours.doc(description="supprimer un contenu en cours")
class SupprimerEnCoursParId(Resource):
    @en_cours.response(200, "Contenu en cours supprimé")
    ## supprimer un contenu en cours
    @jwt_required()
    def delete(self, id_en_cours):
        id_contenu = json.loads(id_en_cours)
        en_cours_dao.supprimer_contenu_en_cours_via_id(id_contenu["$oid"])
        return "Contenu En Cours supprimé", 200


@en_cours.route("/newEnCours")
@en_cours.doc(params={'id_contenu': "l'id du contenu d'origine"})
@en_cours.doc(params={'auteur': "l'auteur du contenu en cours"})
@en_cours.doc(params={'version': "la version du contenu modifié pour créé l'en cours "})
@en_cours.doc(params={'provenance': "la provenance du contenu en cours (perso ou tiers)"})
@en_cours.doc(description="créer un contenu en cours")
class CreerEnCours(Resource):
    @en_cours.response(200, "Contenu en cours créé")
    ## créer un contenu en cours
    @jwt_required()
    def post(self):
        id_contenu = json.loads(request.json["id_contenu"])
        en_cours_dao.creer_en_cours(request.json['auteur'], id_contenu["$oid"], request.json['version'],
                                    request.json['provenance'])

        return json.loads(json.dumps(en_cours_dao.get_contenu_en_cours_via_id(en_cours_dao.recuperer_dernier_id()),
                                     default=json_util.default)), 201


@en_cours.route("/newEnCoursRefuse")
@en_cours.doc(params={'id_contenu_en_cours': "l'id en cours du contenu d'origine de la demande de publication"})
@en_cours.doc(description="créer un contenu en cours refusé, consécutivement à une demande de publication refusée")
class CreerEnCoursRefuse(Resource):
    @en_cours.response(201, "Contenu en cours mis à jour au statut de refusé")
    ## créer un contenu en cours refusé, consécutivement à une demande de publication refusée
    @jwt_required()
    def patch(self):
        id_contenu_en_cours = json.loads(request.json["id_contenu_en_cours"])

        en_cours_dao.creer_en_cours_refuse(id_contenu_en_cours["$oid"])
        return "En Cours Refuse créé", 201


@en_cours.route("/changerTitre")
@en_cours.doc(params={'id_contenu_en_cours': "l'id du contenu en cours concerné"})
@en_cours.doc(params={'titre': "le nouveau titre"})
@en_cours.doc(description="ajouter ou modifier un titre")
class ModifierTitre(Resource):
    @en_cours.response(201, "Titre du contenu en cours modifié")
    def patch(self):
        id_contenu_en_cours = json.loads(request.json["id_contenu_en_cours"])
        en_cours_dao.changer_titre(id_contenu_en_cours["$oid"], request.json["titre"])
        return "Titre modifié", 201


demande_pub = api.namespace('demande_pub', description='Entrées liées à la gestion des demandes de publication')


## *********************************************************************************************************************
## Entrées liées à la gestion des demandes de publication

@demande_pub.route("/demandePublication")
@demande_pub.doc(params={'id_contenu': "l'id du contenu associé à la demande de publication"})
@demande_pub.doc(params={'id_contenu_en_cours': "l'id du contenu en cours associé à la demande de publica"})
@demande_pub.doc(params={'auteur_en_cours': "l''auteur de la demande de publica"})
@demande_pub.doc(params={'version': "la version 'effective' du contenu publié"})
@demande_pub.doc(description="créer une demande de publication")
class demande_de_publication(Resource):
    @demande_pub.response(201, "Demande de publication créée")
    ## créer une demande de publication
    @jwt_required()
    def post(self):
        id_contenu = json.loads(request.json["id_contenu"])
        id_contenu_en_cours = json.loads(request.json["id_contenu_en_cours"])

        auteur = en_cours_dao.get_auteur_origine_contenu_en_cours(id_contenu_en_cours["$oid"])
        demande_pub_Dao.creer_demande_publication(id_contenu["$oid"], id_contenu_en_cours["$oid"],
                                                  request.json['auteur_en_cours'], auteur, request.json["version"])
        # return "Demande de publication créée", 201

        return json.loads(json.dumps((demande_pub_Dao.get_demande_pub_via_id(demande_pub_Dao.recuperer_dernier_id())),
                                     default=json_util.default)), 201


@demande_pub.route("/demandePublication/<auteur>/<emise>")
@demande_pub.doc(params={'emise': "si true -> publications proposees, si false -> publications recues"})
@demande_pub.doc(description="récupérer les demandes de publication pour un auteur, proposées ou recues")
class get_demandes_pub(Resource):
    @demande_pub.response(200, "Demande de publication récupérées pour l'auteur renseigné")
    ## récupérer les demandes de publication pour un auteur, proposées ou recues
    @jwt_required()
    def get(self, auteur, emise):
        return json.loads(json.dumps(list(demande_pub_Dao.get_demandes_pub(auteur, distutils.util.strtobool(emise))),
                                     default=json_util.default))


@demande_pub.route("/demandePublication/<id_publication>")
@demande_pub.doc(description="supprimer une demande de publication")
class supprimer_demande_pub(Resource):
    @demande_pub.response(200, "Demande de publication supprimée")
    ## supprimer une demande de publication
    @jwt_required()
    def delete(self, id_publication):
        id_publication = json.loads(id_publication)

        demande_pub_Dao.supprimer_publication(id_publication["$oid"])
        return "Demande de publication supprimée"


evaluations = api.namespace('evaluations', description='Entrées liées à la gestion des evaluations')


## *********************************************************************************************************************
## Entrées liées à la gestion des évaluations

@evaluations.route("/newEval", methods=['POST'])
@evaluations.doc(params={'contenus': "les contenus associés à la création d'une evaluation"})
@evaluations.doc(params={'auteur': "l'auteur de l'évaluation"})
@evaluations.doc(description="créer une évaluation")
class creer_evaluation(Resource):
    @evaluations.response(201, "Evaluation créée")
    ## créer une évaluation
    @jwt_required()
    def post(self):
        evaluation_dao.creer_evaluation(request.args.get('contenus'), request.args.get('auteur'))
        return "evaluation créée", 201


@evaluations.route("/getEval/<auteur>/<id_evaluation>", methods=['GET'])
@evaluations.doc(description="récupérer une évaluation d'un auteur via son id")
class get_evaluation(Resource):
    @evaluations.response(200, "Evaluation récupérée")
    ## récupérer une évaluation d'un auteur via son id
    @jwt_required()
    def get(self, auteur, id_evaluation):
        return json.loads(
            json.dumps(list(evaluation_dao.get_evaluations(auteur, id_evaluation)), default=json_util.default)), 200


@evaluations.route("/getEval/<auteur>", methods=['GET'])
@evaluations.doc(description="récupérer toutes les évaluations d'un auteur")
class get_all_evaluations(Resource):
    @evaluations.response(200, "Toutes les évaluations récupérées")
    ## récupérer toutes les évaluations d'un auteur
    @jwt_required()
    def get(self, auteur):
        return json.loads(json.dumps(list(evaluation_dao.get_evaluations(auteur, 0)), default=json_util.default)), 200


@evaluations.route("/deleteEval", methods=['DELETE'])
@evaluations.doc(description="supprimer une évaluation")
class supprimer_evaluation(Resource):
    @evaluations.response(200, "Evaluation supprimée")
    ## supprimer une évaluation
    @jwt_required()
    def delete(self, id_evaluation):
        evaluation_dao.supprimer_evaluation(id_evaluation)
        return "Evaluation supprimée", 200


if __name__ == '__main__':
    app.run(host='0.0.0.0')
    # app.run()
