import json

from bson import ObjectId

from pymongo import MongoClient
from random import randrange
from datetime import datetime
import os
# from UtilsGit import UtilsGit
from pathlib import Path
import string

from werkzeug.utils import secure_filename

from config.gitUtils import GitUtils
from config.utils import Utils
from modele.encours_dao import EncoursDao


class ContenuDao:
    client = MongoClient('localhost', 27017)
    chemin = "/home/OpenQuizz/contenus/"
    gitUtils = GitUtils()

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Fonction permettant d'ajouter un contenu dans le panier de l'auteur
    # @param l'auteur en cours
    # @param nom_fichier le nom du fichier associé au contenu
    # @param commit le 'hash' du commit associé (récupéré via git)
    def ajouter_contenu(self, auteur, contenu_a_ecrire, titre, tags):

        # créer la métadonnée dans la base mongo
        self.base_donnees.contenu.insert_one(
            {"auteur": auteur, "popularite": 0,
             "fichier": "", "titre": titre,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "note": 0, "nbreNotes": 0, "commit": ""},
             "tags_fixes": tags,
             "type": " ", "source": "markdown"})

        nom_contenu = self.id_dernier_contenu_ajoute()

        chemin_tags = '/'.join(tags) + "/"
        chemin_fichier = self.chemin + chemin_tags + nom_contenu + "/"
        Path(chemin_fichier).mkdir(0o777, parents=True, exist_ok=True)

        myFile = open(chemin_fichier + nom_contenu, "w")
        # y ajouter le texte du contenu
        myFile.write(contenu_a_ecrire)
        myFile.close()

        # mettre a jour le chemin du contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                             {"$set": {"fichier": chemin_tags + nom_contenu + "/" + nom_contenu}})

        # initialiser le git et récupérer le commit
        commit = self.gitUtils.commit(chemin_fichier, nom_contenu, "version" + str(1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                             {"$set": {"versionEnCours.commit": commit}})

    ## Fonction qui permet d'importer un nouveau contenu
    def importer_contenu(self, auteur, file):

        from xml.dom import minidom

        global chemin_fichier, filename
        if file:
            filename = secure_filename(file.filename)

            print(filename)

            chemin_fichier = self.chemin + filename + "/"
            Path(chemin_fichier).mkdir(0o777, parents=True, exist_ok=True)
            file.save(os.path.join(chemin_fichier, filename))

            print(chemin_fichier + filename)

            xmldoc = minidom.parse(chemin_fichier + filename)
            mite = xmldoc.getElementsByTagName('name')
            titre = mite[0].childNodes[1].firstChild.nodeValue

        # créer la métadonnée dans la base mongo
        self.base_donnees.contenu.insert_one(
            {"auteur": auteur, "popularite": 0,
             "fichier": filename + "/" + filename, "titre": titre,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "note": 0, "nbreNotes": 0, "commit": ""},
             "tags_fixes": ["tag1", "tag2", "tag3", "tag4", "tag5"],
             "type": "quiz", "source": "moodle"})

        nom_contenu = self.id_dernier_contenu_ajoute()

        # initialiser le git et récupérer le commit
        commit = self.gitUtils.commit(chemin_fichier, filename, "version" + str(1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(nom_contenu)},
                                             {"$set": {"versionEnCours.commit": commit}})

    ## récupérer la popularité d'une version
    # @param id_contenu l'id du contenu
    # @param version le numéro de version
    def get_popularite(self, id_contenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["popularite"]

    ## incrémenter la popularité d'un contenu ou d'une de ses versions
    # @param id_contenu l'id du contenu
    # @param version le numéro de version incrémenté
    def incrementer_popularite(self, id_contenu):

        popularite = self.get_popularite(id_contenu)

        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                             {"$set": {"popularite": popularite + 1}})

    ## recherche d'un contenu par id (pour affichage de son contenu. charge la version courante
    # @param id_contenu l'id du contenu recherché
    def get_contenu_par_id(self, id_contenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})

    ## Récupérer l'id du dernier contenu ajouté
    # @return l'id du dernier contenu (sous la forme d'une chaine de caracteres)
    def id_dernier_contenu_ajoute(self):
        # créer un fichier (au bon endroit)
        # récupérer l'id mongo (pour nommer le répertoire)
        tmp = self.base_donnees.contenu.find().sort("_id", -1).limit(1)

        record = next(tmp, None)
        return str(record.get("_id"))

    ## recherche des contenus pour un auteur donné (son 'panier')
    # @param auteur l'auteur des contenus
    # @param perso true si contenus persos a charger, false sinon
    # @param tiers true si contenus tiers a charger, false sinon
    # @param en_vours true si contenus en cours a charger, false sinon
    # @return tous les contenus présents dans le panier de l'auteur
    def get_contenus(self, auteur, perso, tiers, en_cours):

        contenus = list()

        if tiers:
            id_tiers = self.get_contenus_tiers(auteur)
            for i in range(len(id_tiers)):
                contenu_tiers = self.get_contenu_par_id(id_tiers[i][0]["contenu"])
                contenu_tiers["tags_libres"] = self.recuperer_tags_libres_contenu(id_tiers[i][0]["contenu"], auteur)
                contenu_tiers["provenance"] = "tiers"
                contenu_tiers["version"] = id_tiers[i][1]["version"]
                note = self.recupererNoteContenu(id_tiers[i][0]["contenu"], id_tiers[i][1]["version"], auteur)
                if note == -1:
                    contenu_tiers["note_perso"] = "-"
                    # print("prout caca : " + contenu_tiers["note_perso"])
                else:
                    contenu_tiers["note_perso"] = note
                contenus.append(contenu_tiers)

        if perso:
            curseur_perso = self.get_contenus_persos(auteur)
            for contenu in curseur_perso:
                contenu["provenance"] = "perso"
                contenu["tags_libres"] = self.recuperer_tags_libres_contenu(contenu["_id"], auteur)
                contenus.append(contenu)

        if en_cours:
            curseur_en_cours = EncoursDao().get_contenus_en_cours_auteur(auteur)
            for contenu in curseur_en_cours:
                contenu["provenance"] = "en-cours"
                contenu["tags_libres"] = self.recuperer_tags_libres_contenu(contenu["_id"], auteur)
                # print("tags libres en_cours", contenu["tags_libres"])
                contenus.append(contenu)
            # on garde le développement de l'évaliation pour plus tard
            # return contenus_persos + contenus_en_cours + contenus_tiers
        return contenus

    ## FOnction permettant de récupérer les contenus tiers d'un auteur
    # @param auteur l'auteur en question
    # return les contenus tiers de l'auteur (_id et version)
    def get_contenus_tiers(self, auteur):
        return self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"]

    ## Fonction permettant de charger tous les contenus 'publiés' n'appartenant pas à auteur (pour afficher sa banque)
    # @param auteur l'auteur en cours
    # @return tous les contenus non présents dans le panier de l'auteur
    def get_contenus_non_auteur(self, auteur):
        contenus_tiers = self.get_contenus_tiers(auteur)
        tiers_auteur = list()
        for tier in contenus_tiers:
            tiers_auteur.append(ObjectId(tier[0]["contenu"]))

        return self.base_donnees.contenu.find({"$and": [{"auteur": {"$ne": auteur}}, {"_id": {"$nin": tiers_auteur}}]})

    ## FOnction de création d'un tiers
    # @param auteur l'auteur en cours
    # @param id_contenu le contenu à ajouter en tiers
    # @param version la version dans laquelle le contenu a été pris
    def creer_tiers(self, auteur, id_contenu, version):
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {
            "$push": {
                "tiers": [{"contenu": ObjectId(id_contenu)}, {"version": version}]}})
        self.incrementer_popularite(id_contenu)

    ## FOnction permettant de récupérer les contenus persos d'un auteur
    # @param auteur l'auteur en question
    # return tous les contenus personnels de l'auteur
    def get_contenus_persos(self, auteur):
        return self.base_donnees.contenu.find({"auteur": auteur})

    ## l'auteur 'renie' son contenu, qui reste disponible dans le panier de ceux qui le possédent
    # @param auteur l'id du contenu à renier
    def dissocier_contenu(self, id_contenu):
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"auteur": "Aucun"}})

    ## permet de, soit renier, soit supprimer définitivement un auteur (et le fichier de contenu associé)
    # @param id_contenu l'id du contenu à supprimer
    def supprimer_contenu(self, id_contenu):
        import shutil

        if self.existe_en_tiers(id_contenu) != 0:
            self.dissocier_contenu(id_contenu)

        else:
            chemin = self.get_chemin_du_contenu(id_contenu)
            repertoire = chemin[:chemin.rfind("/")]
            shutil.rmtree(repertoire)
            self.base_donnees.contenu.delete_one({"_id": ObjectId(id_contenu)})

    ## suppression défintive d'un tiers (de la référence)
    # @param id_tiers l'id du tiers à supprimer
    # @param auteur l'auteur concerné
    # @param version la version du tiers à supprimer
    # @param la note associee au contenu
    def supprimer_tiers(self, id_tiers, auteur, version, note):

        # on distingue le cas ou une note a été donnée ou non
        if note == "-":
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": version}]}})

        else:
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": version},
                                                                   {"note": int(note)}]
                                                     }})

        # si le contenu n'est plus possédé par personne, on le supprime définitivement
        if self.is_anonyme(id_tiers) and self.existe_en_tiers(id_tiers) == 0:
            self.supprimer_contenu(id_tiers)

    ## fonction qui permet de savoir le nombre de fois qu'un contenu est présent en 'contenu tiers' chez d'autres auteurs
    # @param auteur l'id du contenu à 'tester'
    # @return le nombre de contenus tiers associés
    def existe_en_tiers(self, id_contenu):

        return self.base_donnees.utilisateur.count_documents(
            {"tiers": {"$elemMatch": {"$elemMatch": {"contenu": ObjectId(id_contenu)}}}})

    ## fonction qui permet de créer un contenu à partir d'une contenu en cours
    # @param chemin le chemin du contenu
    # @param auteur_en_cours l'auteur du contenu en cours
    # @param id_contenu_en_cours l'id du contenu en cours
    def creer_nouveau_contenu(self, id_demande_publication, auteur_en_cours):
        from modele.demande_pubDao import DemandePubDao

        # on récupère l'id du contenu en cours associés à la demande de publication
        id_contenu_en_cours = DemandePubDao().get_id_contenu_en_cours_associe_a_une_publication(
            id_demande_publication)

        # ce dernier permettant de récupérer le texte à écrire dans le contenu nouvellement créé
        contenu_a_ecrire = Utils().recuperer_contenu_fichier(id_contenu_en_cours, True)

        # on utilise la fonction déja existante pour créer le contenu
        self.ajouter_contenu(auteur_en_cours, contenu_a_ecrire, "", [])

        # et on détruit la demande de publication et le contenu en cours associé, pour finir
        DemandePubDao().supprimer_publication(id_demande_publication)
        EncoursDao().supprimer_contenu_en_cours_via_id(id_contenu_en_cours)

    ## récupère le chemin d'un contenu
    # @param id_contenu l'id du contenu recherché
    # @return le chemin désiré
    def get_chemin_du_contenu(self, id_contenu):

        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        return self.chemin + self.base_donnees.contenu.find_one({"_id": id_contenu})["fichier"]

    ## récupère la version en cours du contenu
    # @param id_contenu l'id du contenu
    # @return la version courante du contenu
    def get_version_courante(self, id_contenu):
        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)
        return self.base_donnees.contenu.find_one({"_id": id_contenu})["versionEnCours"]["numero"]

    ## récupère la version d'un contenu mis en tiers (ou 0 si ce contenu n'est pas sossédé en tiers)
    # @param auteur l'auteur concerné
    # @param id_contenu le contenu recherché
    # @return la version du tiers id_contenu (ou 0)
    def get_version_tiers_avec_id(self, id_contenu, auteur):
        tiers = (self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"])
        for i in range(len(tiers)):
            tmp = tiers[i]
            if tmp[0]["contenu"] == ObjectId(id_contenu):
                return tmp[1]["version"]
        return 0

    ## mise à jour d'un contenu
    # @param idContenu l'id du contenu à mettre à jour
    # @param nomBranche le nom de la branche
    # @param auteur l'auteur du contenu à mettre à jour
    # @param enCours permet de déterminer si l'origine de la maj est un contenu en cours ou une demande de publication

    def maj_contenu(self, id_contenu_en_cours, auteur=None, enCours=True):
        from modele.demande_pubDao import DemandePubDao

        if (enCours):

            id_contenu = EncoursDao().get_id_origine_contenu_en_cours(id_contenu_en_cours)

        else:

            id_contenu = DemandePubDao().get_id_contenu_associe_a_une_publication(id_contenu_en_cours)
            id_contenu_en_cours = DemandePubDao().get_id_contenu_en_cours_associe_a_une_publication(id_contenu_en_cours)

        chemin = self.get_chemin_du_contenu(id_contenu)

        repertoire = chemin[:chemin.rfind("/") + 1]
        version = int(self.get_version_courante(id_contenu)) + 1

        # Mise à jour du répertoire git
        # idCommit = UtilsGit.mergeVersion(repertoire, nomBranche)

        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        # Mise à jour des métadonnées - ok
        version_en_cours = self.base_donnees.contenu.find_one({"_id": id_contenu})["versionEnCours"]
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {"$push": {"historique": version_en_cours}})
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {
            "$set": {"versionEnCours": {"numero": version, "date": datetime.now().strftime("%d/%m/%y"), "note": 0,
                                        "nbreNotes": 0,
                                        "commit": ""}}})

        # Si un auteur est passé en paramètres de la fonction maj = on l'ajoute dans les métadonnées
        if auteur is not None:
            self.base_donnees.contenu.update_one({"_id": id_contenu}, {
                "$set": {"auteur": auteur}})
            note = self.recupererNoteContenu(id_contenu, version - 1, auteur)
            self.supprimer_tiers(id_contenu, auteur, version - 1, note)

        # on modifie le texte du contenu d'origine
        nouveau_texte = Utils().recuperer_contenu_fichier(id_contenu_en_cours, True)

        titre_en_cours = EncoursDao().get_titre_en_cours(id_contenu_en_cours)

        # print ("Titre en cours " + titre_en_cours)

        Utils().changer_texte_fichier(id_contenu, nouveau_texte, titre_en_cours, True)

        ## on supprime le contenu en cours
        EncoursDao().supprimer_contenu_en_cours_via_id(id_contenu_en_cours)

        # initialiser le git et récupérer le commit
        version = self.get_version_courante(id_contenu)
        index = chemin.rindex("/")
        chemin = chemin[:index]
        commit = self.gitUtils.commit(chemin, str(id_contenu), "version" + str(version + 1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {"$set": {"versionEnCours.commit": commit}})

    ## lecture d'un contenu (version basique pour les tests)
    # @param id l'id du contenu à consulter
    # @param auteur l'auteur souhaitant consulter le contenu
    def consulter_un_contenu(self, id, auteur):
        # Créer un répertoire au nom de l'auteur (ou l'utiliser si il existe déja)
        auteur = auteur.lower().replace(" ", "_")
        repertoire_auteur = self.chemin + auteur + "/"
        Path(repertoire_auteur).mkdir(0o755, parents=True, exist_ok=True)
        # Copier le fichier lié au contenu dans ce répertoire
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("cp '" + chemin + "'  '" + repertoire_auteur + nom_fichier + "'")
        # self.affichage(chemin)

    ## après fin de consultation, destruction du fichier temporaire généré
    # @param id l'id du contenu consulté
    # @param auteur l'auteur auant effectué la consultation
    def detruire_fichier_tmp(self, id, auteur):
        auteur = auteur.lower().replace(" ", "_") + "/"
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("rm '" + self.chemin + auteur + nom_fichier + "'")

    ## Fonction permettant de déterminer si un contenu est "anonyme"
    # @param idCOntenu le contenu à tester
    # @return booléen true.false si le contenu est anonyme ou pas
    def is_anonyme(self, idContenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(idContenu)})["auteur"] == "Aucun"

    ## Récupérer tous les contenus
    # @return tous les contenus
    def get_tous_les_contenus(self):
        return self.base_donnees.contenu.find()

    # FOnction permettant de déterminer si un certain contenu fait partie des tiers d'un auteur
    # @param auteur l'auteur concerné
    # @param idContenu le contenu à 'tester'
    # @return un booléen correspondant à savoir si le contenu est chez l'auteur ou pas
    def is_tiers_auteur(self, auteur, idContenu):
        return self.get_version_tiers_avec_id(ObjectId(idContenu), auteur) != 0

    ## mettre à jour un contenu 'tiers'
    # @param auteur l'auteur concerné
    # @param id_favori le favori à maj
    # @param nouvelle_version le numero de la nouvelle version
    # @param refuse indique si la mise a jour est acceptee ou refusee
    def maj_tiers(self, id_tiers, nouvelle_version, auteur, refuse=False):
        if not refuse:

            # on vire le précédent
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": nouvelle_version - 1}]}})

            # et on ajoute la version maj
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$push": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": nouvelle_version}]}})
        else:
            # on vire le précédent
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$pull": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": nouvelle_version - 1}]}})

            # et on ajoute la version maj
            self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                     {"$push": {
                                                         "tiers": [{"contenu": ObjectId(id_tiers)},
                                                                   {"version": str(nouvelle_version - 1) + 'r'}]}})

    ## Permet de savoir si au moins un tiers est mettable à jour pour un auteur
    # @param auteur l'auteur concerné
    # @return un booléen indiquant si on a des mises à jour faisables
    def is_tiers_maj(self, auteur):
        # on cherche toutes les versions en cours dans les tiers et on compare avec celles de la banque

        # On réfcupère tous les ties d'un auteur
        tiers = self.get_contenus_tiers(auteur)

        for i in range(len(tiers)):
            versionCourante = self.get_version_courante(tiers[i][0]["contenu"])
            if tiers[i][1]["version"] < versionCourante:
                return True

        return False

    ## Permet de savoir si au moins un tiers est mettable à jour pour un auteur
    # @param auteur l'auteur concerné
    # @return la liste des tiers mettables à jour
    def liste_tiers_maj(self, auteur):
        liste = list()
        # On réfcupère tous les ties d'un auteur
        tiers = self.get_contenus_tiers(auteur)
        for i in range(len(tiers)):
            versionCourante = self.get_version_courante(tiers[i][0]["contenu"])
            if tiers[i][1]["version"] < versionCourante:
                liste.append(self.get_contenu_par_id(tiers[i][0]["contenu"]))

        return liste

    ## Fonction permettant de récupérer le 'hash' d'un commit
    # @param id_contenu l'id du contenu concerné
    # @param version la version pour laquelle on veux récupérer le commit
    # @return le commit associé à la version
    def get_commit_pour_une_version(self, id_contenu, version):

        if version == str(self.get_version_courante(id_contenu)) or version == '0':
            courante = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]
            return courante["commit"]
        else:
            historique = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["historique"]
            for elt in historique:
                if str(elt["numero"]) == version:
                    return elt["commit"]

    ## Fonction permettant de récupérer les tags associés à un contenu
    # @param id_contenu l'id du contenu concerné
    # @return les tags associés au contenu
    def get_tags_contenu(self, id_contenu):
        return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["tags_fixes"]

    ## Mise à jour des tags fixes associés au contenu
    # @param id_contenu l'id du contenu concerné
    # @param tags les nouveaux tags
    def changer_tags_contenus(self, id_contenu, tags):

        ancien_chemin = self.get_chemin_du_contenu(id_contenu)

        # on modifie les tags
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"tags_fixes": tags}})
        # et le chemin du fichier
        chemin_tags = '/'.join(tags) + "/"

        # mettre a jour le chemin du contenu
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                             {"$set": {"fichier": chemin_tags + id_contenu + "/" + id_contenu}})

        # print(self.get_chemin_du_contenu(id_contenu))
        Path(self.chemin + chemin_tags + id_contenu + "/").mkdir(0o777, parents=True, exist_ok=True)
        print(ancien_chemin[:ancien_chemin.rfind("/")])
        print(ancien_chemin[:ancien_chemin.rfind("/") + 1])

        Path(ancien_chemin).rename(self.get_chemin_du_contenu(id_contenu))
        self.rm_tree(Path(ancien_chemin[:ancien_chemin.rfind("/") + 1]))
        # Path(ancien_chemin[:ancien_chemin.rfind("/") + 1]).rmdir()

    ## Fonction utilitaire -> supprimer un repertoire
    # @param path le chemin du repertoire
    def rm_tree(self, pth):
        for child in pth.iterdir():
            if child.is_file():
                child.unlink()
            else:
                self.rm_tree(child)
        pth.rmdir()

    ## Modification d'un titre d'un contenu personnel
    # @param id_en_cours l'id du contenu
    # @param nouveau_titre le titre à changer
    def changer_titre(self, id_en_cours, nouveau_titre):
        self.base_donnees.contenu.update_one({"_id": ObjectId(id_en_cours)}, {"$set": {"titre": nouveau_titre}})

    ## Fonction permettant de créer un tag libre et de l'associer ou non à un contenu
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag créé
    # @param id_contenu l'id du contenu associé (potentiellement vide)
    def creer_tag_libre(self, auteur, tag):

        self.base_donnees.utilisateur.update_one({"nom": auteur}, {
            "$push": {
                "tags": [{"tag": tag}, {"contenus": []}]}})

    ## Fonction permettant de supprimer un tag libre
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag a supprimer
    # @param id_contenu l'id du contenu associé (potentiellement vide)
    def supprimer_tag_libre(self, auteur, mon_tag):
        # on récupère les tags
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        # on "vide" les tags (qu'on réinjectera)
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": []}})

        for tag in tags:
            if tag[0]['tag'] == mon_tag:
                tags.remove(tag)

        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": tags}})

    ## Fonction permettant de dissocier un contenu d'un tag libre
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag concerné
    # @param id_contenu l'id du contenu à dissocier
    def dissocier_tag_libre(self, auteur, mon_tag, id_contenu):
        # on récupère les tags
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        # on "vide" les tags (qu'on réinjectera)
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": []}})

        # on parcourt chaque tag et on vire les mentions à id_contenu
        for tag in tags:
            if tag[0]['tag'] == mon_tag:
                tag[1]['contenus'].remove(ObjectId(id_contenu))
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": tags}})

    ## Fonction permettant d'associer un contenu à un tag libre
    # @param auteur l'auteur du nouveau tag
    # @param tag le tag concerné
    # @param id_contenu l'id du contenu à associer au tag
    def associer_tag_libre(self, auteur, tag, id_contenu):

        # on récupère les tags
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        print("foinest : ", tags)
        # on "vide" les tags (qu'on réinjectera)
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": []}})
        # on parcourt chaque tag et on vire les mentions à id_contenu
        for letag in tags:
            print("Coutch")
            if letag[0]['tag'] == tag:
                letag[1]['contenus'].append(ObjectId(id_contenu))
                # print("J'ajoute le contenu : ", ObjectId(id_contenu), "\n")
                letag[1]['contenus'] = list(set(letag[1]['contenus']))
        self.base_donnees.utilisateur.update_one({"nom": auteur}, {"$set": {"tags": tags}})

    ## Fonction permettant de récupérer tous les tags libres d'un auteur
    # @param auteur l'auteur du nouveau tag
    # @return les tags libres de l'auteur
    def recuperer_all_tags_libres(self, auteur):
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        tags_auteur = list()
        for f in tags:
            tags_auteur.append(f[0]['tag'])
        return tags_auteur

    # recuperer_tags_libres_contenu
    ## Fonction permettant de récupérer les tags libres d'un auteur associés à un contenu en particulier
    # @param auteur l'auteur du nouveau tag
    # @param id_contenu le contenu associé
    # @return les tags libres de l'auteur
    def recuperer_tags_libres_contenu(self, id_contenu, auteur):
        tags = self.base_donnees.utilisateur.find_one({"nom": auteur})["tags"]
        tags_auteur = list()
        for f in tags:
            for cont in f[1]['contenus']:
                if cont == ObjectId(id_contenu):
                    tags_auteur.append(f[0]['tag'])
        return tags_auteur

    ## Fonction permettant de noter un contenu (pour une version donnée)
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    # @param auteur l'auteur qui met la note
    # @param note la note affectée au contenu
    def noterContenuTiers(self, id_contenu, version, auteur, note):

        # on vire le précédent
        self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                 {"$pull": {
                                                     "tiers": [{"contenu": ObjectId(id_contenu)},
                                                               {"version": version}]}})

        # et on ajoute la version maj
        self.base_donnees.utilisateur.update_one({"nom": auteur},
                                                 {"$push": {
                                                     "tiers": [{"contenu": ObjectId(id_contenu)},
                                                               {"version": version},
                                                               {"note": note}]}})

        # et on modifie le contenu de base (sa moyenne)
        return self.modifierMoyenneContenu(id_contenu, version, note)

    ## Fonction permettant de récupérer une note donnée
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    # @param auteur l'auteur qui met la note
    def recupererNoteContenu(self, id_contenu, version, auteur):

        if type(id_contenu) == str:
            id_contenu = ObjectId(id_contenu)

        if type(version) == str:
            version = int(version)

        contenus_tiers = self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"]
        for contenu in contenus_tiers:

            if len(contenu) == 3 and contenu[0]["contenu"] == id_contenu and contenu[1]["version"] == version:
                return json.dumps(contenu[2]["note"])

        return '-'

    ## Fonction permettant de récupérer une note globale pour une version d'un contenu
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    def recupererNoteGlobaleContenu(self, id_contenu, version):
        if int(version) == self.get_version_courante(id_contenu):
            return self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]["note"] \
                   / self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]["nbreNotes"]
        else:
            historique = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["historique"]
            return "popapopapa"

    ## Fonction permettant de changer la moyenne d'une version d'un contenu à l'insertion d'une nouvelle note
    # @param id_contenu l'id du contenu concerné
    # @param version la version du contenu notée
    # @param note la note affectée au contenu
    def modifierMoyenneContenu(self, id_contenu, version, note):
        if version == self.get_version_courante(id_contenu):
            versionEnCours = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["versionEnCours"]
            self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"versionEnCours": []}})
            versionEnCours['note'] = versionEnCours['note'] + note
            versionEnCours['nbreNotes'] = versionEnCours['nbreNotes'] + 1
            self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                                 {"$set": {"versionEnCours": versionEnCours}})

            return json.dumps([versionEnCours['note'], versionEnCours['nbreNotes']])


        else:
            print("Non, là")
            contenus = self.base_donnees.contenu.find_one({"_id": ObjectId(id_contenu)})["historique"]
            self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)}, {"$set": {"historique": []}})
            for cont in contenus:
                if cont['numero'] == version:
                    cont['note'] = cont['note'] + note
                    cont['nbreNotes'] = cont['nbreNotes'] + 1
                    self.base_donnees.contenu.update_one({"_id": ObjectId(id_contenu)},
                                                         {"$set": {"historique": contenus}})
                    return json.dumps([cont['note'], cont['nbreNotes']])
